﻿//////////////////////////////////////////////
// CameraManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Camera ;
	class RenderContext ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT CameraManager final : public nkCommon::SingletonClass<CameraManager>
	{
		friend class nkCommon::SingletonClass<CameraManager> ;

		private :
			
			// Attributes
			// Les caméras en mémoire
			std::unordered_map<std::string, Camera*> _memory ;
				
			// Cam active
			Camera* _activeCam ;
			// Cam de culling
			Camera* _activeCullingCam ;
			// Cam de LOD
			Camera* _activeLodCam ;
			// Cam active par défaut pour tous les contextes si on trouve pas
			Camera* _defaultCam ;
			// Cam active pour le contexte set lors du tick
			Camera* _activeContextCam ;
			// Cam active pour un RenderContext bien donné
			std::unordered_map<RenderContext*, Camera*> _camPerContext ;
			std::unordered_map<RenderContext*, Camera*> _cullingCamPerContext ;
			std::unordered_map<RenderContext*, Camera*> _lodCamPerContext ;

			// Mutex
			std::mutex _mutex ;
		
		private :

			// Functions
			// Constructor
			CameraManager () ;
		
		public :

			// Destructor
			~CameraManager () ;
		
			// Manipulation mémoire
			Camera* createOrRetrieve (const std::string_view& name) ;
			Camera* get (const std::string_view& name) ;
			Camera* getByIndex (unsigned int index) ;
			void rename (const std::string_view& currentName, const std::string_view& newName) ;
			void erase (const std::string_view& name) ;

			// Getters
			Camera* getActiveRenderCam () const ;
			Camera* getActiveCullingCam () const ;
			Camera* getActiveLodCam () const ;
			Camera* getDefaultCam () const ;
			Camera* getActiveContextCam () const ;
			Camera* getContextRenderCam (RenderContext* context) ;
			Camera* getContextCullingCam (RenderContext* context) ;
			Camera* getContextLodCam (RenderContext* context) ;

			// Setters
			void setActiveRenderCam (Camera* cam) ;
			void setActiveCullingCam (Camera* cam) ;
			void setActiveLodCam (Camera* cam) ;
			void setDefaultCam (Camera* cam) ;
			void setActiveContextCam (Camera* cam) ;
			void setContextRenderCam (RenderContext* context, Camera* cam) ;
			void setContextCullingCam (RenderContext* context, Camera* cam) ;
			void setContextLodCam (RenderContext* context, Camera*cam) ;

			// Resets lors rendu
			void resetRenderCamToActiveContext () ;
			void resetCullingCamToActiveContext () ;
			void resetLodCamToActiveContext () ;
			void resetRenderCamToDefault () ;
			void resetCullingCamToDefault () ;
			void resetLodCamToDefault () ;

			// Préparation
			void prepareForContext (RenderContext* context) ;
	} ;
}