﻿//////////////////////////////////////////////
// ResourceManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class ResourceSaveLoadCallback ;
	class ResourceSetLoadQuery ;
	class ResourceSetLoadResult ;
	class ResourceSetSaveQuery ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// Common
#include <nilkinsCommon/Patterns/SingletonClass.h>

// Standards
#include <string_view>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ResourceManager : public nkCommon::SingletonClass<ResourceManager>
	{
		// Singleton ami
		friend class nkCommon::SingletonClass<ResourceManager> ;

		private :

			// Constructeur, destructeur
			ResourceManager () ;
			~ResourceManager () ;
		
		public :

			// Chargement de ressource
			void loadResourceSetDescFileSync (const ResourceSetLoadQuery& query, ResourceSetLoadResult& resultRef) ;
			void loadResourceSetDescFileAsync (const ResourceSetLoadQuery& query, ResourceSaveLoadCallback* callback = nullptr) ;
			// Sauvegarde
			void saveResourceSetDescFile (const ResourceSetSaveQuery& query) ;

			// Requêtes de chargement en particulier
			// Ici il y aura gestion pour ne pas locker le thread de rendu, ou le moins possible
			// ContentLoader reste disponible pour chargements plus raw (synchrones)
			// Caméras
			void loadCamera (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// States
			void loadBlendState (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			void loadDepthStencilState (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			void loadRasterState (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// Textures
			void loadSampler (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			void loadTexture (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// Buffer
			void loadBuffer (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// Shaders
			void loadShader (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			void loadProgram (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// Mesh
			void loadMesh (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
			// Compositor
			void loadCompositor (const std::string_view& path, const std::string_view& compoToDefault = "", ResourceSaveLoadCallback* callback = nullptr) ;
			// Node
			void loadNode (const std::string_view& path, ResourceSaveLoadCallback* callback = nullptr) ;
	} ;
}