﻿//////////////////////////////////////////////
// ImageBase.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkImages
#include "../Dll/DllDefines.h"

#include "../Enums/PixelFormat.h"

// nkMaths
#include <NilkinsMaths/Algebra/Vector.h>

/// Class ------------------------------------

namespace nkImages
{
	class DLL_IMAGES_EXPORT ImageBase
	{
		private :

			// Attributes
			// Image characteristics
			unsigned int _width ;
			unsigned int _height ;
			unsigned int _depth ;
			PIXEL_FORMAT _format ;
			
			// Alignment in memory
			unsigned int _pixelByteSize ;
			unsigned int _rowByteSize ;
			unsigned int _sliceByteSize ;

		public :

			// Constructor, destructor
			ImageBase () ;
			ImageBase (unsigned int width, unsigned int height, PIXEL_FORMAT format) ;
			ImageBase (unsigned int width, unsigned int height, unsigned int depth, PIXEL_FORMAT format, unsigned int pixelByteSize, unsigned int rowByteSize) ;
			ImageBase (const ImageBase& other) ;
			ImageBase (ImageBase&& other) ;
			virtual ~ImageBase () ;

			// Getters
			unsigned int getWidth () const ;
			unsigned int getHeight () const ;
			unsigned int getDepth () const ;
			PIXEL_FORMAT getFormat () const ;
			unsigned int getPixelByteSize () const ;
			unsigned int getRowByteSize () const ;
			unsigned int getSliceByteSize () const ;

			// Setters
			void setWidth (unsigned int value) ;
			void setHeight (unsigned int value) ;
			void setDepth (unsigned int value) ;
			void setFormat (PIXEL_FORMAT value) ;
			void setPixelByteSize (unsigned int value) ;
			void setRowByteSize (unsigned int value) ;
			void setSliceByteSize (unsigned int value) ;

			// Utils
			nkMaths::Vector getPixel (unsigned int x, unsigned int y, unsigned int z = 1) ;

			// Data accessors
			virtual unsigned char* getDataPtr () const = 0 ;

			// Operators
			ImageBase& operator= (const ImageBase& other) ;
			ImageBase& operator= (ImageBase&& other) ;
	} ;
}