﻿//////////////////////////////////////////////
// ListBox.h
//
//////////////////////////////////////////////

/// Defines / macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "Component.h"

#include "../Dll/DllDefines.h"

#include "../Inputs/CallbackRepository.h"

// Standards
#include <map>
#include <string>
#include <unordered_map>

/// Class ------------------------------------

namespace nkWinUi
{
	class DLL_WINUI_EXPORT ListBox : public Component
	{
		public :

			// Les items rangés dedans
			class DLL_WINUI_EXPORT Item
			{
				private :

					// Attributs
					ListBox* _parentList ;
					std::string _label ;

					void* _attachedData ;

				public :

					// Constructeur, destructeur
					Item () ;
					~Item () ;

					// Getters
					ListBox* getParentList () const ;
					const char* getLabel () const ;
					void* getAttachedData () const ;

					// Setters
					void setParentList (ListBox* parent) ;
					void setLabel (const std::string_view& label) ;
					void setAttachedData (void* data) ;
			} ;

		protected :
		
			// Attributs
			std::map<std::string, Item*, std::less<>> _memory ;
			std::unordered_map<void*, Item*> _memoryPerData ;

			// Sélection active
			Item* _selected ;
				
			// Callback
			listBoxSelectionCallback _callback ;

			// Height de ligne
			unsigned int _lineHeight ;

		public :
		
			// Constructeur, destructeur
			ListBox () ;
			~ListBox () ;

			// Getters
			Item* getItemFromLabel (const std::string_view& label) const ;
			Item* getItemFromData (void* data) const ;
			Item* getItemFromIndex (unsigned int index) const ;
			virtual Item* getCurrentSelection () const ;
			virtual int getCurrentSelectionIndex () const ;
			listBoxSelectionCallback getSelectionSpecializedCallback () const ;
			unsigned int getItemCount () const ;

			// Setters
			void setSelectionSpecializedCallback (listBoxSelectionCallback value) ;

			// Manipulation d'Items
			virtual Item* insertItem (const std::string_view& text, void* data) ;
			virtual void* deleteItem (const std::string_view& name) ;
			virtual void changeItemLabel (const std::string_view& oldText, const std::string_view& newText) ;
			void changeItemData (const std::string_view& text, void* newData) ;
			virtual bool setSelected (const std::string_view& text) ;
			virtual bool setSelectedIndex (unsigned int index) ;
			bool setSelectedData (void* data) ;
			virtual void clearItems () ;

			// Import / Export
			virtual void exportIntrospection (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}