﻿//////////////////////////////////////////////
// GridLayout.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

#include "Layout.h"

// nkMaths
#include <NilkinsMaths/Geometry/Point.h>

// Standards
#include <unordered_map>
#include <vector>

/// Internals --------------------------------

namespace nkWinUi
{
	struct GridLayoutEntryInfo
	{
		Component* _component ;

		unsigned int _position [2] ;
		unsigned int _size [2] ;
	} ;
}

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT GridLayout final : public Layout
	{
		private :

			// Nos components associés à leurs cases
			std::vector<GridLayoutEntryInfo> _grid ;

			// Map des cases à taille fixe
			std::unordered_map<unsigned int, float> _fixedSizeRows ;
			std::unordered_map<unsigned int, float> _fixedSizeCols ;

			// Nos infos sur notre grille
			unsigned int _size [2] ;

			// Internal layout info
			nkMaths::Point _internalBorderSize ;
		
		public :
		
			// Constructeur, destructeur
			GridLayout () ;
			~GridLayout () ;

			// Getters
			Component* getComponentAt (unsigned int index) ;

			// Setters
			virtual void setParentComponent (Component* parent) override ;
			void setGridWidth (unsigned int value) ;
			void setGridHeight (unsigned int value) ;
			void setFixedSizeRow (unsigned int rowIndex, float sizePix) ;
			void setFixedSizeCol (unsigned int colIndex, float sizePix) ;
			void setInternalBorderWidth (float value) ;
			void setInternalBorderHeight (float value) ;

			// Components
			void addComponent (Component* component, unsigned int x, unsigned int y, unsigned int width, unsigned int height) ;
			virtual void removeComponent (Component* component) override ;

			// Update
			virtual void update () override ;

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void exportComponentsNodeEntry (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
			virtual void processComponentsNodeEntry (nkExport::Node* rootNode) override ;
	} ;
}